#if defined __APPLE__
#include "SamplesOSX.h"
#else
#include "sample.h"
#endif
#include "sample_common.h"
#include "sprk.h"

#include "sprk_exchange.h"

#include <iostream>

using namespace HPS;

void Sample::Exchange_Component_Highlighting(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
{
	try
	{
	
	/* Sample: Exchange_Component_Highlighting
	 * This sample will show you how to:
	 * - Walk the component structure of a CAD Model
	 * - Highlight a specific instance of a component
	 * - Highlight all instances of a component
	 * - Build a KeyPath
	 * - Define a Named Style used for highlighting
	 * 
	 * Read more about the topics covered in this sample in our Programming Guide 
	 * - Integrating HPS with HOOPS Exchange
	 * - Section 6.2:		Highlighting
	 * - Section 1.2.1:		Using Keys and Objects
	 * */

	/* This next block of code shows you how to load an Exchange file
	 * 1. Create an ImportOptionsKit
	 *		NOTE: It is required to import BRep data to access geometry referenced by PMI
	 * 2. Use the Exchange::File::Import method to load the desired file, with the options you specified
	 * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	 * 4. Use a try-catch block to capture any Exchange IO exceptions 
	 * 5. Obtain a CADModel object through the Exchange::ImportNotifier */
	CADModel cadModel;
	Exchange::ImportNotifier exchangeNotifier;
	try {
		UTF8 file_in = INPUT_FOLDER + "/LandingGear.prc";
		Exchange::ImportOptionsKit importOptionsKit;
		importOptionsKit.SetBRepData(true);
		exchangeNotifier = Exchange::File::Import(static_cast<const char*>(file_in), importOptionsKit);
		exchangeNotifier.Wait();

		cadModel = exchangeNotifier.GetCADModel();
	}

	catch(IOException ioe) {
		UTF8 error_message;
		error_message = "Exchange Import Unsuccessful";
		modelKey.InsertText(Point(0, 0, 0), static_cast<const char*>(error_message));
		view.FitWorld();
		view.Update();
	}

	/* This block of code sets up a named style which will be used for highlighting
	 * 1. Obtain a portfolio key. The portfolio used will need to be visible from the segment
	 *		using the name style defined in it.
	 * 2. Create a segment which will contain the attributes for highlighting 
	 * 3. Use the DefineNamedStyle method to create a named style in a Portfolio
	 *		To create a named style, provide it with a name, and specify the segment key
	 *		of the segment containing the attributes used for highlighting */
	PortfolioKey pKey;
	view.GetSegmentKey().GetPortfolioControl().ShowTop(pKey);

	SegmentKey redSegment = view.GetSegmentKey().Subsegment();
	redSegment.GetMaterialMappingControl().SetFaceColor(RGBColor(1, 0, 0))
		.SetTextColor(RGBColor(1, 0.65f, 0))
		.SetLineColor(RGBColor(1, 0.65f, 0));
	pKey.DefineNamedStyle("redHighlight", redSegment);

	/* This block of code sets up our scene:
	 * We will have two subwindows, each including an instance of the CAD Model we have imported:
	 * 1. Activate the default capture from the CAD Model
	 * 2. Include the default capture into two segments, these two segments will house the two subwindows */
	SegmentKey view1, view2;
	view1 = view.GetSegmentKey().Subsegment();
	view2 = view.GetSegmentKey().Subsegment();
	View defaultView = cadModel.ActivateDefaultCapture();
	defaultView.FitWorld();
	IncludeKey view1Include = view1.IncludeSegment(defaultView.GetSegmentKey());
	IncludeKey view2Include = view2.IncludeSegment(defaultView.GetSegmentKey());

	view1.GetSubwindowControl().SetSubwindow(HPS::Rectangle(-1, 0, -1, 1)).SetBorder(Subwindow::Border::Inset);
	view1.InsertDistantLight(Vector(1, 1, 0));
	view2.GetSubwindowControl().SetSubwindow(HPS::Rectangle(0, 1, -1, 1)).SetBorder(Subwindow::Border::Inset);
	view2.InsertDistantLight(Vector(1, 1, 0));
	
	view.Update();

	/* This code shows how to walk the component structure of the CAD Model
	 * For this particular model we assume that we know where the data we wish to highlight is located
	 * The model loaded in this file represents the landing gear of an aeroplane.
	 * The landing gear contains four wheels. 
	 * Only one wheel prototype exists in the model, and four instances of it are then included
	 * where appropriate to represent four wheels without replicating data.
	 * 
	 * A key path to the specific part will be needed, and can be obtained using the GetKeyPath method */
	ComponentArray rootComponent = cadModel.GetSubcomponents();
	ComponentArray parts = rootComponent[0].GetSubcomponents();
	ComponentArray wheels = parts[2].GetSubcomponents();
	ComponentArray wheelInstance = wheels[1].GetSubcomponents();

	KeyPathArray wheelKeyPath;
	wheelKeyPath = Component::GetKeyPath(wheelInstance[0]);

	/* This next block of code performs highlighting 
	 * The KeyPathArray object obtained above will contain four KeyPath objects:
	 * each one describing the path to one of the four instances of a wheel.
	 * 
	 * To highlight all instances of a wheel we complete the path all the way to
	 * the window for all KeyPath objects, and highlight them */
	HighlightOptionsKit highlightOptions;
	highlightOptions.SetStyleName("redHighlight");

	for (auto kp : wheelKeyPath)
	{
		KeyArray keys;
		kp.ShowKeys(keys);
		keys.push_back(view1Include);
		keys.push_back(canvas.GetAttachedLayout().GetAttachedViewIncludeLink(0));
		keys.push_back(canvas.GetAttachedLayoutIncludeLink());
		keys.push_back(wk);

		KeyPath highlightKeyPath(keys);
		wk.GetHighlightControl().Highlight(highlightKeyPath, highlightOptions);
	}

	/* This code shows how to highlight only one instance of a component
	 * To highlight only one component we repeat the actions performed above, but this
	 * time, they are performed only on the specific KeyPath of interest. */
	KeyPath oneWheelPath = wheelKeyPath[0];
	KeyArray keys;
	oneWheelPath.ShowKeys(keys);
	keys.push_back(view2Include);
	keys.push_back(canvas.GetAttachedLayout().GetAttachedViewIncludeLink(0));
	keys.push_back(canvas.GetAttachedLayoutIncludeLink());
	keys.push_back(wk);

	KeyPath highlightKeyPath(keys);
	wk.GetHighlightControl().Highlight(highlightKeyPath, highlightOptions);

	/* The next block adds text description to both subwindows. For each subwindow the following
	 * operations are performed:
	 * 1. A subsegment is created, and depth range is set on it to make sure that the geometry in this
	 *		segment will always appear on top
	 * 2. Text visibility is enabled through the VisibilityControl
	 * 3. A stretched camera is added to the subsegment
	 * 4. The text attributes are modified as needed using the TextAttributeControl
	 * 5. Text is inserted */
	SegmentKey view1Description = view1.Subsegment();
	view1Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
	view1Description.GetVisibilityControl().SetEverything(false).SetText(true);
	view1Description.GetCameraControl().SetPosition(Point(0, 0, -5)).
										SetTarget(Point(0, 0, 0)).
										SetUpVector(Vector(0, 1, 0)).
										SetField(2, 2).
										SetProjection(Camera::Projection::Stretched);
	view1Description.GetTextAttributeControl().SetSize(0.02f, Text::SizeUnits::SubscreenRelative).
										SetAlignment(Text::Alignment::Center, Text::ReferenceFrame::WorldAligned, Text::Justification::CenterJustified);
	view1Description.InsertText(Point(0, 0.85f, 0), "All instances of the component\n are highlighted in red");


	SegmentKey view2Description = view2.Subsegment();
	view2Description.GetDrawingAttributeControl().SetDepthRange(0.0f, 0.1f);
	view2Description.GetVisibilityControl().SetEverything(false).SetText(true);
	view2Description.GetCameraControl().SetPosition(Point(0, 0, -5)).
		SetTarget(Point(0, 0, 0)).
		SetUpVector(Vector(0, 1, 0)).
		SetField(2, 2).
		SetProjection(Camera::Projection::Stretched);
	view2Description.GetTextAttributeControl().SetSize(0.02f, Text::SizeUnits::SubscreenRelative).
		SetAlignment(Text::Alignment::Center, Text::ReferenceFrame::WorldAligned, Text::Justification::CenterJustified);
	view2Description.InsertText(Point(0, 0.85f, 0), "Component instance is highlighted in red");
	

	}
	catch(IOException e)
	{
		std::cout << e.what() << std::endl;
	}
}
